// packages/@code-like-a-carpenter/foundation-cli/src/schema.ts
import { readFileSync } from "fs";
import path from "node:path";
import { getCachedDocumentNodeFromSchema } from "@graphql-codegen/plugin-helpers";
import { mergeSchemas } from "@graphql-tools/schema";
import { globSync } from "glob";
import {
  buildASTSchema,
  extendSchema,
  lexicographicSortSchema,
  parse,
  Source
} from "graphql";
function getDocumentNodeFromAddToSchemaResult(a) {
  if (typeof a === "string") {
    return parse(a);
  }
  if (typeof a === "undefined") {
    return parse("");
  }
  return a;
}
function makeInitialSchema(addToSchemaResults) {
  const [first, ...rest] = Array.from(new Set(addToSchemaResults));
  return rest.reduce(
    (acc, next) => extendSchema(acc, getDocumentNodeFromAddToSchemaResult(next)),
    buildASTSchema(getDocumentNodeFromAddToSchemaResult(first))
  );
}
async function loadSchema(cwd, schemaGlob, addToSchemaResults) {
  const initialSchema = makeInitialSchema(addToSchemaResults);
  const patterns = Array.isArray(schemaGlob) ? schemaGlob : [schemaGlob];
  const files = globSync(patterns, { cwd });
  const sources = files.map(
    (file) => new Source(readFileSync(path.resolve(cwd, file), "utf8"), file)
  );
  const typeDefs = sources.map((source) => parse(source, {}));
  return getCachedDocumentNodeFromSchema(
    lexicographicSortSchema(
      mergeSchemas({ typeDefs: [initialSchema, ...typeDefs] })
    )
  );
}
export {
  loadSchema
};
